import os
import re
import sys
from contextlib import contextmanager

import click
import click_pathlib
import kconfiglib

from listconfig import print_tree


@contextmanager
def quietify(verbose):
    if verbose:
        yield None
        return

    try:
        with open(os.devnull, 'w') as devnull:
            sys.stderr = devnull
            yield None
    finally:
        sys.stderr = sys.__stderr__


@click.command()
@click.argument('kconfig_path', type=click_pathlib.Path(exists=True))
@click.argument('dotconfig_path', type=click_pathlib.Path(exists=True))
@click.option('--arch', default=None, help='ARCH in Linux kernel. Inferred by default.')
@click.option('--help-lines', '-l', default=2, help='Number of lines to pick from a config help.')
@click.option('--verbose', '-v', is_flag=True, help='Enable warnings during Kconfig analysis.')
def main(kconfig_path, dotconfig_path, arch, help_lines, verbose):
    if not kconfig_path.is_file():
        print(f'Specified Kconfig path {kconfig_path} is not a file', file=sys.stderr)
        return sys.exit(1)

    ksrc = kconfig_path.parent

    if 'srctree' not in os.environ:
        os.environ['srctree'] = str(ksrc)

    if 'CC' not in os.environ:
        os.environ['CC'] = 'gcc'

    if arch is None:
        r = re.compile('Linux/([^ ]+) ')
        with open(dotconfig_path, 'r') as f:
            detected = None

            for l in f.readlines()[:5]:
                res = r.search(l)
                if res is None:
                    continue
                detected = res.groups()[0]

            if detected is None:
                print('Specified .config has no expected header', file=sys.stderr)
                print('Please specify --arch={arch} manually', file=sys.stderr)
                sys.exit(1)

            arch = detected

    if 'SRCARCH' not in os.environ:
        os.environ['SRCARCH'] = os.environ.get('ARCH', arch)

    with quietify(verbose):
        kconfig = kconfiglib.Kconfig(str(kconfig_path))

    if not dotconfig_path.is_file():
        print(f'Specified .config path {dotconfig_path} is not a file', file=sys.stderr)

    with quietify(verbose):
        kconfig.load_config(str(dotconfig_path))

    print_tree(kconfig.top_node.list, help_lines)


main()



import kconfiglib


__version__ = '0.0.1'


def value_str(sc):
    """
    Returns the value part ('[*]', '<M>', '(foo)' etc.) of a menu entry.
    sc: Symbol or Choice.
    """

    if sc.type in (kconfiglib.STRING, kconfiglib.INT, kconfiglib.HEX):
        return f'({sc.str_value})'

    # BOOL or TRISTATE

    # The choice mode is an upper bound on the visibility of choice symbols, so
    # we can check the choice symbols' own visibility to see if the choice is
    # in y mode
    if isinstance(sc, kconfiglib.Symbol) and sc.choice and sc.visibility == 2:
        # For choices in y mode, print '-->' next to the selected symbol
        return ' ->' if sc.choice.selection is sc else '   '

    tri_val_str = (' ', 'M', '*')[sc.tri_value]

    if len(sc.assignable) == 1:
        # Pinned to a single value
        return f'-{tri_val_str}-'

    if sc.type == kconfiglib.BOOL:
        return f'[{tri_val_str}]'

    if sc.type == kconfiglib.TRISTATE:
        if sc.assignable == (1, 2):
            # m and y available
            return f'{{{tri_val_str}}}'
        return f'<{tri_val_str}>'


def node2str(node):
    prompt, cond = node.prompt
    if not kconfiglib.expr_value(cond):
        return None, None

    if node.item == kconfiglib.MENU:
        return f'=== {prompt} ===', None

    if node.item == kconfiglib.COMMENT:
        return f'*** {prompt} ***', None

    # Now node.item is a symbol or a choice
    if node.item.type == kconfiglib.UNKNOWN:
        return None, None

    res = f'{value_str(node.item):3} {prompt}'

    if node.item.name is not None:
        res += f' ({node.item.name})'

    if hasattr(node, 'help') and node.help is not None:
        help_lines = [l for l in node.help.split('\n') if l.strip()]
        return res, help_lines

    return res, None


def dig(node, indent):
    lines = []
    count = 0

    while node:
        if node.prompt:
            s, help_lines = node2str(node)
            if s is not None and not s.startswith('*** Compiler:'):
                count += 1
                lines.append(('  ' * indent + s, help_lines))
                # print('  ' * indent + s)

        if node.list:
            if count == 0:
                lines += dig(node.list, indent)
            else:
                lines += dig(node.list, indent + 1)

        node = node.next

    return lines


def print_tree(node, nlines):
    lines = dig(node, 0)
    longest = max(len(t[0]) for t in lines)

    for t in lines:
        if nlines == 0:
            print(t[0])
            continue

        print(t[0] + ' ' * (longest - len(t[0])), end=' ')
        if t[1] is None:
            print()
        else:
            lines = t[1]
            l = ' '.join(lines[:nlines])
            if len(lines) > nlines:
                l += f' ...'
            print(l)
